////////////////////////////////////////////////////////////////////////////////
///                            MeshMainWindow.h                              ///
////////////////////////////////////////////////////////////////////////////////

#ifndef MESHMAINWINDOW_H
#define MESHMAINWINDOW_H

#include <X3DTK/MESH/scenegraph.h>
#include <MESH_Controller.h>
#include <MESH_MeshController.h>
#include "meshEditorForm.h"
#include "MeshViewer.h"
#include <cmath>
#include <limits>
#include <iostream>
#include <qfiledialog.h>
#include <qmessagebox.h> 
#include <qlayout.h> 
#include <X3DTK/kernel.h>
#include <MESH_MeshSelectionView.h>

namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
class MeshMainWindow : public MainWindow
{
public:
  /// Constructor.
  MeshMainWindow(QWidget* parent = 0, const char* name = 0, WFlags fl = WType_TopLevel);
  
  /// Destructor.
  virtual ~MeshMainWindow();
  
  virtual void fileOpen();
  virtual void fileSave();
  virtual void fileSaveAs();
  virtual void fileExit();
  virtual void focusMeshViewer00();
  virtual void focusMeshViewer10();
  virtual void focusMeshViewer01();
  virtual void focusMeshViewer11();

  virtual void loadFile(const char *fileName);
  virtual void buildMeshScene();

  void addViewToSelection(View *view);

protected:
  MeshController<MData, VData, EData, FData, readOnly> *meshController; 
  
private:
  MeshSelectionView<MData, VData, EData, FData, readOnly> *_meshSelectionView;
  bool _viewer00Focused;
  bool _viewer10Focused;
  bool _viewer01Focused;
  bool _viewer11Focused;
};

}
}

#include "MeshMainWindow.inl"

#endif
