namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
MeshMainWindow<MData, VData, EData, FData, readOnly>::MeshMainWindow(QWidget* parent, const char* name, WFlags fl)
: MainWindow(parent, name, fl), _viewer00Focused(false), _viewer10Focused(false), _viewer01Focused(false), _viewer11Focused(false)
{
  meshController = new MeshController<MData, VData, EData, FData, readOnly>();
  _meshSelectionView = new MeshSelectionView<MData, VData, EData, FData, readOnly>();
  _meshSelectionView->show();
  meshController->recordSelection(_meshSelectionView);
  
  // setting controller
  meshViewer00->setController(meshController);
  meshViewer10->setController(meshController);
  meshViewer01->setController(meshController);
  meshViewer11->setController(meshController);

  RC::Signal::connect(meshViewer00, &MeshViewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer00);
  RC::Signal::connect(meshViewer10, &MeshViewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer10);
  RC::Signal::connect(meshViewer01, &MeshViewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer01);
  RC::Signal::connect(meshViewer11, &MeshViewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer11);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MeshMainWindow<MData, VData, EData, FData, readOnly>::~MeshMainWindow()
{
  meshController->release();
  
  delete meshController;
  delete _meshSelectionView;
  
  RC::Signal::disconnect(meshViewer00, &MeshViewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer00);
  RC::Signal::disconnect(meshViewer10, &MeshViewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer10);
  RC::Signal::disconnect(meshViewer01, &MeshViewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer01);
  RC::Signal::disconnect(meshViewer11, &MeshViewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer11);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshMainWindow<MData, VData, EData, FData, readOnly>::loadFile(const char *fileName)
{
  if (fileName == 0)
    return;

  // Loads the file name and disabling the validation of the file by setting the 
  // 2nd parameter to false.
  meshController->loadFile(fileName);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshMainWindow<MData, VData, EData, FData, readOnly>::buildMeshScene()
{
  meshController->buildMeshScene();

  // QGLViewer settings.
  SFVec3f min = meshController->getBBoxMin();
  SFVec3f max = meshController->getBBoxMax();
  SFVec3f center = 0.5f*(max + min);
  SFVec3f radius = max - center;
  min = center - 2.0f*radius;
  max = center + 2.0f*radius;

  meshViewer00->setSceneBoundingBox(min.f_data(), max.f_data());
  meshViewer01->setSceneBoundingBox(min.f_data(), max.f_data());
  meshViewer10->setSceneBoundingBox(min.f_data(), max.f_data());
  meshViewer11->setSceneBoundingBox(min.f_data(), max.f_data());
  meshViewer00->showEntireScene();
  meshViewer01->showEntireScene();
  meshViewer10->showEntireScene();
  meshViewer11->showEntireScene(); 

  meshController->notify();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshMainWindow<MData, VData, EData, FData, readOnly>::addViewToSelection(View *view)
{
  _meshSelectionView->addView(view);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshMainWindow<MData, VData, EData, FData, readOnly>::fileOpen()
{
  QString fileName = QFileDialog::getOpenFileName("", "X3D files (*.x3d *.X3D);;All files (*)", this);

  // In case of Cancel
  if (fileName.isEmpty())
    return;
  
  loadFile(fileName);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshMainWindow<MData, VData, EData, FData, readOnly>::fileSave()
{
  meshController->save();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshMainWindow<MData, VData, EData, FData, readOnly>::fileSaveAs()
{
  QString fileName = QFileDialog::getSaveFileName("", "X3D files (*.x3d *.X3D)", this);

  // In case of Cancel
  if (fileName.isEmpty())  
    return;
 
  meshController->saveAs(fileName);   
}
  
template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshMainWindow<MData, VData, EData, FData, readOnly>::fileExit()
{
  meshController->release();
  delete meshController;

  exit(1);
}
  
template<class MData, class VData, class EData, class FData, bool readOnly>  
void MeshMainWindow<MData, VData, EData, FData, readOnly>::focusMeshViewer00()
{
  _viewer00Focused = !_viewer00Focused;
  
  if (_viewer00Focused)
  {
    meshViewer10->hide();
    meshViewer01->hide();
    meshViewer11->hide();
    
    MainWindowLayout->setRowStretch(0, 2);
    MainWindowLayout->setColStretch(0, 2);
    MainWindowLayout->setRowStretch(1, 0);
    MainWindowLayout->setColStretch(1, 0);
  }  
  else
  {
    MainWindowLayout->setRowStretch(0, 1);
    MainWindowLayout->setColStretch(0, 1);
    MainWindowLayout->setRowStretch(1, 1);
    MainWindowLayout->setColStretch(1, 1);
    
    meshViewer10->show();
    meshViewer01->show();
    meshViewer11->show();
  }
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshMainWindow<MData, VData, EData, FData, readOnly>::focusMeshViewer10()
{
  _viewer10Focused = !_viewer10Focused;
  
  if (_viewer10Focused)
  {
    meshViewer00->hide();
    meshViewer01->hide();
    meshViewer11->hide();
    
    MainWindowLayout->setRowStretch(0, 2);
    MainWindowLayout->setColStretch(1, 2);
    MainWindowLayout->setRowStretch(1, 0);
    MainWindowLayout->setColStretch(0, 0);
  }  
  else
  {
    MainWindowLayout->setRowStretch(0, 1);
    MainWindowLayout->setColStretch(1, 1);
    MainWindowLayout->setRowStretch(1, 1);
    MainWindowLayout->setColStretch(0, 1);
    
    meshViewer00->show();
    meshViewer01->show();
    meshViewer11->show();
  }
}

template<class MData, class VData, class EData, class FData, bool readOnly>  
void MeshMainWindow<MData, VData, EData, FData, readOnly>::focusMeshViewer01()
{
  _viewer01Focused = !_viewer01Focused;
  
  if (_viewer01Focused)
  {
    meshViewer00->hide();
    meshViewer10->hide();
    meshViewer11->hide();
    
    MainWindowLayout->setRowStretch(1, 2);
    MainWindowLayout->setColStretch(0, 2);
    MainWindowLayout->setRowStretch(0, 0);
    MainWindowLayout->setColStretch(1, 0);
  }  
  else
  {
    MainWindowLayout->setRowStretch(1, 1);
    MainWindowLayout->setColStretch(0, 1);
    MainWindowLayout->setRowStretch(0, 1);
    MainWindowLayout->setColStretch(1, 1);
    
    meshViewer00->show();
    meshViewer10->show();
    meshViewer11->show();
  }
}

template<class MData, class VData, class EData, class FData, bool readOnly>  
void MeshMainWindow<MData, VData, EData, FData, readOnly>::focusMeshViewer11()
{
  _viewer11Focused = !_viewer11Focused;
  
  if (_viewer11Focused)
  {
    meshViewer00->hide();
    meshViewer10->hide();
    meshViewer01->hide();
    
    MainWindowLayout->setRowStretch(1, 2);
    MainWindowLayout->setColStretch(1, 2);
    MainWindowLayout->setRowStretch(0, 0);
    MainWindowLayout->setColStretch(0, 0);
  }  
  else
  {
    MainWindowLayout->setRowStretch(1, 1);
    MainWindowLayout->setColStretch(1, 1);
    MainWindowLayout->setRowStretch(0, 1);
    MainWindowLayout->setColStretch(0, 1);
    
    meshViewer00->show();
    meshViewer10->show();
    meshViewer01->show();
  }
}

}
}
