#include "MeshNormalsDock.h"
#include <qmainwindow.h>
#include <cmath>
#include <X3DTK/kernel.h>

#include <iostream>

using namespace std;

MeshNormalsDock::MeshNormalsDock(QMainWindow *parent)
: QToolBar(parent)
{
  parent->moveDockWindow(this, DockRight);
  
  _form = new NormalsAmplitudeForm(this);
  _form->amplitudeSlider->setRange(0, 1000);
  _form->baseSlider->setRange(0, 1000);
  setCaption(tr("Normals Amplitude"));

  init();
  
  connect(_form->amplitudeSlider, SIGNAL(valueChanged(int)), this, SLOT(setValue()));
  connect(_form->baseSlider, SIGNAL(valueChanged(int)), this, SLOT(setValue()));
  connect(_form->checkBox, SIGNAL(released()), this, SLOT(changeRealTime()));
}

void MeshNormalsDock::init()
{
  _form->amplitudeSlider->setValue(500);
  _form->baseSlider->setValue(500);
}

float MeshNormalsDock::getValue() const 
{
  float m = 10.0f;

  float a = static_cast<float>(_form->amplitudeSlider->value())/static_cast<float>(_form->amplitudeSlider->maxValue());
  float b = static_cast<float>(_form->baseSlider->value())/static_cast<float>(_form->baseSlider->maxValue());
  
  float rb = (pow(m*m, b) - 1.0f)/m;
  float ra = 2.0f*rb*(a - 0.5f);
  
  return ra + rb;
}


void MeshNormalsDock::setValue()
{
  RC::Signal::send(this, &MeshNormalsDock::getValue);
}

void MeshNormalsDock::changeRealTime()
{
  if (_form->checkBox->isChecked())
  {
    disconnect(_form->amplitudeSlider, SIGNAL(sliderReleased()), this, SLOT(setValue()));
    disconnect(_form->baseSlider, SIGNAL(sliderReleased()), this, SLOT(setValue()));
    
    connect(_form->amplitudeSlider, SIGNAL(valueChanged(int)), this, SLOT(setValue()));
    connect(_form->baseSlider, SIGNAL(valueChanged(int)), this, SLOT(setValue()));
  }
  else
  {
    disconnect(_form->amplitudeSlider, SIGNAL(valueChanged(int)), this, SLOT(setValue()));
    disconnect(_form->baseSlider, SIGNAL(valueChanged(int)), this, SLOT(setValue()));

    connect(_form->amplitudeSlider, SIGNAL(sliderReleased()), this, SLOT(setValue()));
    connect(_form->baseSlider, SIGNAL(sliderReleased()), this, SLOT(setValue()));
  }
}
