#ifndef MESHVIEWER_H
#define MESHVIEWER_H

#include <QGLViewer/qglviewer.h>
#include <X3DTK/kernel.h>
#include <list>
#include <map>

namespace X3DTK {
namespace MESH {

class Controller;
class View;

}
}

class MeshViewerMenu;

/// Viewer for the meshEditor.

class MeshViewer : public QGLViewer
{
public:
  /// Constructor.
  MeshViewer(QWidget *parent = 0, const char *name = 0, const QGLWidget *shareWidget = 0, WFlags flags = 0);
  /// Destructor.
  ~MeshViewer();
  
  /// Sets the Mesh controller.
  void setController(X3DTK::MESH::Controller *controller);
  /// Records the view.
  void record(X3DTK::MESH::View *view);
  /// Removes the view.
  void remove(X3DTK::MESH::View *view);
  /// Call display for every view.
  void display();
  /// Call select for every view.
  void select();

  /// Double click signal.
  void doubleClicked() const {};

  /// Sets the item i.
  void setItem(int i);

protected:
  /// init method.
  void init();
  /// draw method.
  void draw();
  /// Select event.
  void select(const QMouseEvent *e);
  /// mouse double click event.
  void mouseDoubleClickEvent(QMouseEvent *e);
  /// mouse right click event.
  void contextMenuEvent(QContextMenuEvent *e);
 
private:
  X3DTK::MESH::Controller *_controller;
  std::list<X3DTK::MESH::View *> _viewList;
  std::map<int, X3DTK::MESH::View *> _viewMap;
  MeshViewerMenu *_contextMenu;
};

#endif
