#ifndef MESHEDITORMAINWINDOW_H
#define MESHEDITORMAINWINDOW_H

#include <X3DTK/MESH/scenegraph.h>
#include <X3DTK/X3D/sharedcoordcomputer.h>
#include <X3DTK/X3D/coordmerger.h>
#include <MESH_EdgesView.h>
#include <MESH_FacesView.h>
#include <MESH_VerticesView.h>
#include <MESH_NormalsView.h>
#include <MESH_NormalsInversionView.h>
#include <MESH_ConnComView.h>
#include <MESH_StatisticsView.h>
#include <PreprocessingDialog.h>

#include "MeshMainWindow.h"

namespace X3DTK {
namespace MESH {

/// MeshEditorMainWindow

template<class MData, class VData, class EData, class FData, bool readOnly>
class MeshEditorMainWindow : public MeshMainWindow<MData, VData, EData, FData, readOnly>
{
public:
  /// Constructor.
  MeshEditorMainWindow(QWidget *parent = 0, const char *name = 0, typename MeshMainWindow<MData, VData, EData, FData, readOnly>::WFlags fl = WType_TopLevel);
  /// Destructor.
  virtual ~MeshEditorMainWindow();
  
  virtual void loadFile(const char *fileName);
  virtual void preprocess();
  
private:
  X3D::SharedCoordComputer *_sharedCoordComputer;
  X3D::CoordMerger *_coordMerger;
  PreprocessingDialog *_dialog;

  FacesView<MData, VData, EData, FData, readOnly> *_facesView; 
  EdgesView<MData, VData, EData, FData, readOnly> *_edgesView;
  VerticesView<MData, VData, EData, FData, readOnly> *_verticesView;
  NormalsInversionView<MData, VData, EData, FData, readOnly> *_normalsInversionView;
  NormalsView<MData, VData, EData, FData, readOnly> *_normalsView;
  ConnComView *_connComView;
  StatisticsView<MData, VData, EData, FData, readOnly> *_statsView;
};

}
}

#include "MeshEditorMainWindow.inl"

#endif
