namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
MeshEditorMainWindow<MData, VData, EData, FData, readOnly>::MeshEditorMainWindow(QWidget *parent, const char *name, typename MeshMainWindow<MData, VData, EData, FData, readOnly>::WFlags fl)
: MeshMainWindow<MData, VData, EData, FData, readOnly>(parent, name, fl)
{
  _sharedCoordComputer = new X3D::SharedCoordComputer();
  _coordMerger = new X3D::CoordMerger();
  _dialog = new PreprocessingDialog();
  RC::Signal::connect(_dialog, &PreprocessingDialog::compute, this, &MeshEditorMainWindow<MData, VData, EData, FData, readOnly>::preprocess);
   
  _facesView = new FacesView<MData, VData, EData, FData, readOnly>(this); 
  _edgesView = new EdgesView<MData, VData, EData, FData, readOnly>(); 
  _verticesView = new VerticesView<MData, VData, EData, FData, readOnly>();
  _normalsInversionView = new NormalsInversionView<MData, VData, EData, FData, readOnly>(this); 
  _normalsView = new NormalsView<MData, VData, EData, FData, readOnly>(this); 
  _connComView = new ConnComView(this);
  _statsView = new StatisticsView<MData, VData, EData, FData, readOnly>(this);
        
  addViewToSelection(_verticesView);
  addViewToSelection(_edgesView);
  addViewToSelection(_facesView);
  addViewToSelection(_normalsInversionView);
  addViewToSelection(_connComView);
  addViewToSelection(_statsView);
    
  meshViewer00->record(_facesView);
  meshViewer00->record(_edgesView);
  meshViewer00->record(_normalsInversionView);
  meshViewer00->record(_connComView);
  meshViewer00->record(_statsView);
  meshViewer10->record(_verticesView);
  meshViewer00->record(_normalsView);
    
  _facesView->show();
  _edgesView->show();
  _verticesView->show();
  _normalsInversionView->show();
  _normalsView->show();
  _connComView->show();
  _statsView->show();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MeshEditorMainWindow<MData, VData, EData, FData, readOnly>::~MeshEditorMainWindow()
{
  delete _sharedCoordComputer;
  delete _coordMerger;

  RC::Signal::disconnect(_dialog, &PreprocessingDialog::compute, this, &MeshEditorMainWindow<MData, VData, EData, FData, readOnly>::preprocess);
  delete _dialog;
  
  delete _facesView; 
  delete _edgesView;   
  delete _verticesView;
  delete _normalsView;
  delete _normalsInversionView;
  delete _connComView;
  delete _statsView;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshEditorMainWindow<MData, VData, EData, FData, readOnly>::loadFile(const char *fileName)
{
  if (fileName != 0)
  {
    MeshMainWindow<MData, VData, EData, FData, readOnly>::loadFile(fileName);
    _dialog->show();
  }  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshEditorMainWindow<MData, VData, EData, FData, readOnly>::preprocess()
{
  _dialog->hide();

  if (_dialog->sharedCoord())
    _sharedCoordComputer->compute(meshController->getX3DScene());
  
  if (_dialog->mergedCoord())
    _coordMerger->merge(meshController->getX3DScene());  

  MeshMainWindow<MData, VData, EData, FData, readOnly>::buildMeshScene();
}

}
}
