#ifndef TEMPLATECONTROLLER_H
#define TEMPLATECONTROLLER_H

#include "MESH_Controller.h"

#include <X3DTK/kernel.h>
#include <X3DTK/X3D/meshbuilder.h>
#include <X3DTK/memreleaser.h>
#include <X3DTK/X3D/scenegraph.h>
#include <X3DTK/X3D/bboxupdater.h>
#include <X3DTK/X3D/scenesaver.h>
#include <X3DTK/MESH/scenegraph.h>
#include <X3DTK/MESH/normalsupdater.h>
#include <X3DTK/MESH/x3dbuilder.h>
#include <list>

namespace X3DTK {
namespace MESH {

/// class providing a controller for the MeshViewer MVC pattern.

template<class MData, class VData, class EData, class FData, bool readOnly>
class MeshController : public Controller
{
public:
  /// Constructor.
  MeshController();
  /// Destructor.
  virtual ~MeshController();
  
  /// Loads the file.
  virtual void load(const char *file);
  /// Saves the scene.
  void save();
  /// Saves the scene as the file.
  void saveAs(const char *file);
  /// Releases the memory.
  void release();

  /// Gets the Bbox min.
  inline const SFVec3f &getBBoxMin() const;
  /// Gets the Bbox max.
  inline const SFVec3f &getBBoxMax() const;

  /// Gets the X3D scene.
  X3D::Scene *getX3DScene() const;
 
  /// Loads the X3D scene.
  void loadFile(const char *file);
  /// Computes the BBox of the X3D scene.
  void computeBBox();
  /// Builds the Mesh scene.
  void buildMeshScene();   

private:
  X3D::Scene *_x3dScene;
  
  X3D::Loader *_loader;
  X3D::BBoxUpdater *_bboxupdater;
  X3D::SceneSaver *_saver;
  MemReleaser *_releaser;
    
  X3D::TemplateMeshBuilder<MData, VData, EData, FData, readOnly> *_builder;
  TemplateX3DBuilder<MData, VData, EData, FData, readOnly> *_x3dbuilder;
  TemplateNormalsUpdater<MData, VData, EData, FData, readOnly> *_normalsUpdater;
  SFVec3f _min;
  SFVec3f _max;
};

}
}

#include "MESH_MeshController.inl"

#endif
