#include <X3DTK/graphtester.h>

namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
MeshController<MData, VData, EData, FData, readOnly>::MeshController()
: _x3dScene(0), _min(SFVec3f(0.0f, 0.0f, 0.0f)), _max(SFVec3f(0.0f, 0.0f, 0.0f))
{
  _loader = Singleton<X3D::Loader>::getInstance();
  _bboxupdater = Singleton<X3D::BBoxUpdater>::getInstance();
  _saver = Singleton<X3D::SceneSaver>::getInstance();
  _releaser = Singleton<MemReleaser>::getInstance();
  _builder = Singleton<X3D::TemplateMeshBuilder<MData, VData, EData, FData, readOnly> >::getInstance();
  _x3dbuilder = Singleton<TemplateX3DBuilder<MData, VData, EData, FData, readOnly> >::getInstance();
  _normalsUpdater = Singleton<TemplateNormalsUpdater<MData, VData, EData, FData, readOnly> >::getInstance();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MeshController<MData, VData, EData, FData, readOnly>::~MeshController()
{
  release();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshController<MData, VData, EData, FData, readOnly>::load(const char *file)
{
  loadFile(file);
  buildMeshScene();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshController<MData, VData, EData, FData, readOnly>::save()
{
  _x3dScene = _x3dbuilder->build(meshScene);
  _saver->save(_x3dScene);
  _releaser->release(_x3dScene);
  _x3dScene = 0;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshController<MData, VData, EData, FData, readOnly>::saveAs(const char *file)
{
  _x3dScene = _x3dbuilder->build(meshScene);
  _saver->saveAs(_x3dScene, file);
  _releaser->release(_x3dScene);
  _x3dScene = 0;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshController<MData, VData, EData, FData, readOnly>::release()
{
  _releaser->release(meshScene);
  meshScene = 0;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const SFVec3f &MeshController<MData, VData, EData, FData, readOnly>::getBBoxMin() const 
{
  return _min;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const SFVec3f &MeshController<MData, VData, EData, FData, readOnly>::getBBoxMax() const 
{
  return _max;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
X3D::Scene *MeshController<MData, VData, EData, FData, readOnly>::getX3DScene() const
{
  return _x3dScene;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshController<MData, VData, EData, FData, readOnly>::loadFile(const char *file)
{
  release();
  _x3dScene = _loader->load(file, false);
  computeBBox(); 
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshController<MData, VData, EData, FData, readOnly>::computeBBox()
{
  _bboxupdater->update(_x3dScene, true);

  if (_x3dScene != 0)
  {
    SFVec3f center = _x3dScene->getBBoxCenter();
    SFVec3f size = _x3dScene->getBBoxSize();

    SFVec3f A = center + 0.5f*size;
    SFVec3f B = center - 0.5f*size;

    if (A.x < B.x)
    {
      _min.x = A.x;
      _max.x = B.x;
    }  
    else
    {
      _min.x = B.x;  
      _max.x = A.x;
    }  

    if (A.y < B.y)
    {
      _min.y = A.y;
      _max.y = B.y;
    }  
    else
    {
      _min.y = B.y;  
      _max.y = A.y;
    }

    if (A.z < B.z)
    {
      _min.z = A.z;
      _max.z = B.z;
    }  
    else
    {
      _min.z = B.z;  
      _max.z = A.z;
    }  
  }  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MeshController<MData, VData, EData, FData, readOnly>::buildMeshScene()
{
  meshScene = _builder->build(_x3dScene);
  _normalsUpdater->updateNormals(meshScene);
  _releaser->release(_x3dScene);
  _x3dScene = 0;
}

}
}
