#ifndef MESHVIEW_H
#define MESHVIEW_H

#include "MESH_Controller.h"
#include <X3DTK/MESH/scenegraph.h>
#include <X3DTK/kernel.h>

#ifdef max
  #undef max
#endif  

namespace X3DTK {
namespace MESH {

/// Class providing an interface for a Mesh view.

class View
{
  friend class Controller;
public:
  /// Constructor.
  View(const SFString &name);
  /// Destructor.
  virtual ~View();

  /// Hides the view.
  virtual void hide();
  /// Shows the view.
  virtual void show();
  /// Notifies the controller that the content of the Scene has changed.
  void notify();
  /// Receives notification from the controller that the content of the Scene has changed.
  virtual void receiveNotification();
  /// Displays the content.
  virtual void display();
  /// Initializes the selection. This method has to be redefined if select is defined.
  virtual void initSelect();
  /// Select.
  virtual void select();
  /// Sets the selected nodes to draw. If empty, then all the graph is drawn.
  virtual void setSelectedNodes(const MFNode &selectedNodes);
  
  /// Returns true if visible.
  inline bool isVisible() const {return _isVisible;};
  /// Signal.
  void updated() const {};
  
  /// Gets the name.
  inline const SFString &getName() const {return _name;};

  /// Gets the Scene.
  Scene *getScene() const;

private:
  SFString _name;  
  Controller *_controller;
  bool _isVisible;
};

}
}

#endif
