#include "MESH_ConnComView.h"
#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

ConnComView::ConnComView(QMainWindow *parent) 
: View("connected components"), dock(new MeshConnComDock(parent))
{
  _connComComputer = Singleton<ConnectedComComputer>::getInstance();
  RC::Signal::connect(dock, &MeshConnComDock::compute, this, &ConnComView::compute);
}

ConnComView::~ConnComView()
{
  RC::Signal::disconnect(dock, &MeshConnComDock::compute, this, &ConnComView::compute);
}

void ConnComView::hide()
{
  View::hide();
  dock->hide();
}

void ConnComView::show()
{
  View::show();
  dock->show();
}

void ConnComView::compute()
{
  QProgressDialog *progress = new QProgressDialog("Working...", "Cancel", 0, 0, "Work progress", true);
  progress->setAutoClose(false);
  progress->show();
  
  RC::Signal::connect(_connComComputer->progress, &Progress::getTotalSteps, progress, &QProgressDialog::setTotalSteps);
  RC::Signal::connect(_connComComputer->progress, &Progress::getProgress, progress, &QProgressDialog::setProgress);

  _connComComputer->compute(getScene());

  notify();

  RC::Signal::disconnect(_connComComputer->progress, &Progress::getTotalSteps, progress, &QProgressDialog::setTotalSteps);
  RC::Signal::disconnect(_connComComputer->progress, &Progress::getProgress, progress, &QProgressDialog::setProgress);

  delete progress;
}

void ConnComView::setSelectedNodes(const MFNode &selectedNodes)
{
  _connComComputer->setSelectedNodes(selectedNodes);
}

}
}
