#ifndef MESHEDGESVIEW_H
#define MESHEDGESVIEW_H

#include <MESH_View.h>
#include <X3DTK/MESH/scenegraph.h>
#include <X3DTK/MESH/edgesdrawer.h>
#include <qobject.h>
#include <limits>
#include <GL/gl.h>

namespace X3DTK {
namespace MESH {

/// View for the edges.

template<class MData, class VData, class EData, class FData, bool RW>
class EdgesView : public View
{
public:
  /// Constructor.
  EdgesView();
  /// Destructor.
  virtual ~EdgesView();
  
  /// Receives notification from the controller that the content of the Scene has changed.
  virtual void receiveNotification();  
  /// Displays the content.
  virtual void display();
  /// Initializes the selection. This method has to be redefined if select is defined.
  virtual void initSelect();
  /// Displays the informations necessary to the selection.
  virtual void select();  
  /// Gets the value of the selection.
  unsigned int getValue() const;
  
  /// Gets the current selected Edge.
  inline SFTemplateEdge<MData, VData, EData, FData, RW> *getSelectedEdge() const;
  /// Gets the current selected Vertex.
  inline TemplateVertex<MData, VData, EData, FData, RW> *getSelectedVertex() const;
  /// Gets the current selected Matrix.
  inline const SFMatrix34f &getSelectedTransform() const;

private:
  TemplateEdgesDrawer<MData, VData, EData, FData, RW> *_edgesDrawer;
  bool _modified;
  bool _modifiedSelection;
  GLuint _displayList;
  GLuint _selectionDisplayList;
  unsigned int _zMin;
  TemplateVertex<MData, VData, EData, FData, RW> *_selectedVertex;
  SFTemplateEdge<MData, VData, EData, FData, RW> *_selectedEdge;
  SFMatrix34f _selectedTransform;
};

}
}

#include "MESH_EdgesView.inl"

#endif
