#ifndef MESHFACESVIEW_H
#define MESHFACESVIEW_H

#include <MESH_View.h>
#include <X3DTK/MESH/scenegraph.h>
#include <X3DTK/MESH/facesdrawer.h>
#include <limits>
#include <GL/gl.h>
#include <iostream>
#include "MeshFacesDock.h"

namespace X3DTK {
namespace MESH {

/// View for the faces.

template<class MData, class VData, class EData, class FData, bool RW>
class FacesView : public View
{
public:
  /// Dock window for the graphical interface.
  MeshFacesDock *dock;
  
  /// Constructor.
  FacesView(QMainWindow *parent);
  /// Destructor.
  virtual ~FacesView();
  
  /// Receives notification from the controller that the content of the Scene has changed.
  virtual void receiveNotification();
  /// Displays the content.
  virtual void display();
  /// Initializes the selection. This method has to be redefined if select is defined.
  virtual void initSelect();
  /// Displays the informations necessary to the selection.
  virtual void select();
  /// Gets the value of the selection.
  unsigned int getValue() const;
  
  /// Gets the current selected Face.
  inline SFTemplateFace<MData, VData, EData, FData, RW> *getSelectedFace() const;
  /// Gets the current selected Mesh.
  inline TemplateMesh<MData, VData, EData, FData, RW> *getSelectedMesh() const;
  /// Gets the current selected Matrix.
  inline const SFMatrix34f &getSelectedTransform() const;
  /// Sets the selected nodes to draw. If empty, then all the graph is drawn.
  void setSelectedNodes(const MFNode &selectedNodes);

  /// Changes the rendering to real.
  void real();
  /// Changes the rendering to the indexed faced sets.
  void indexedFaceSet();
  /// Changes the rendering to the connected components.
  void connComs();

private:
  TemplateFacesDrawer<MData, VData, EData, FData, RW> *_facesDrawer;
  bool _modified;
  bool _modifiedSelection;  
  GLuint _displayList;
  GLuint _selectionDisplayList;
  unsigned int _zMin;
  TemplateMesh<MData, VData, EData, FData, RW> *_selectedMesh;
  SFTemplateFace<MData, VData, EData, FData, RW> *_selectedFace;
  SFMatrix34f _selectedTransform;
};

}
}

#include "MESH_FacesView.inl"

#endif
