#ifndef MESH_MESHSELECTIONVIEW_H
#define MESH_MESHSELECTIONVIEW_H

#include <MESH_View.h>
#include <X3DTK/MESH/scenegraph.h>
#include <X3DTK/kernel.h>
#include <list>
#include <GL/gl.h>
#include <MESH_EdgesView.h>
#include <MESH_FacesView.h>
#include <MESH_VerticesView.h>
#include <algorithm>

namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
class MeshSelectionView : public View
{
public:
  /// Constructor.
  MeshSelectionView();
  /// Destructor.
  virtual ~MeshSelectionView();  
  
  /// Receives notification from the controller that the content of the Scene has changed.
  virtual void receiveNotification();
  /// Select.
  virtual void select();
  
  /// Gets the list of selected nodes.
  MFNode getSelectedNodes() const;
  
  /// Adds a View.
  void addView(View *view);
  /// Removes a View.
  void removeView(View *view);
    
private:  
  std::list<View *> _viewList;
  VerticesView<MData, VData, EData, FData, RW> *_verticesView;
  EdgesView<MData, VData, EData, FData, RW> *_edgesView;
  FacesView<MData, VData, EData, FData, RW> *_facesView;
  
  SFTemplateFace<MData, VData, EData, FData, RW> *_face;
  SFTemplateEdge<MData, VData, EData, FData, RW> *_edge;
  SFTemplateVertex<MData, VData, EData, FData, RW> *_vertex;
  SFMatrix34f _transform;
  MFNode _meshList;
};

}
}

#include "MESH_MeshSelectionView.inl"

#endif
