namespace X3DTK {
namespace MESH {

typedef enum {NIL, VERTEX, EDGE, FACE} Entity;

template<class MData, class VData, class EData, class FData, bool RW>
MeshSelectionView<MData, VData, EData, FData, RW>::MeshSelectionView()
: View("selection"), _verticesView(0), _edgesView(0), _facesView(0),
_face(0), _edge(0), _vertex(0), _transform(SFMatrix34f::identity)
{
}

template<class MData, class VData, class EData, class FData, bool RW>
MeshSelectionView<MData, VData, EData, FData, RW>::~MeshSelectionView()
{
}

template<class MData, class VData, class EData, class FData, bool RW>  
void MeshSelectionView<MData, VData, EData, FData, RW>::receiveNotification()
{
}

template<class MData, class VData, class EData, class FData, bool RW>
void MeshSelectionView<MData, VData, EData, FData, RW>::select()
{
  Entity entity = NIL;
  unsigned int value = std::numeric_limits<unsigned int>::max();
  unsigned int currentValue = 0;
  
  if ((_verticesView != 0) 
   && (_verticesView->isVisible()) 
   && (_verticesView->getSelectedVertex() != 0))
  { 
    currentValue = _verticesView->getValue();
    if (currentValue != std::numeric_limits<unsigned int>::max())
      currentValue -= (1 << 24);
    if (currentValue < value)
    {
      entity = VERTEX;
      value = currentValue;
    }  
  }
    
  if ((_edgesView != 0) 
   && (_edgesView->isVisible())
   && (_edgesView->getSelectedEdge() != 0)) 
  {
    currentValue = _edgesView->getValue();
    if (currentValue < value)
    {
      entity = EDGE;
      value = currentValue;
    } 
  }
  
  if ((_facesView != 0)
   && (_facesView->isVisible())
   && (_facesView->getSelectedFace() != 0))
  {
    currentValue = _facesView->getValue();
    if (currentValue < value)
    {
      entity = FACE;
      value = currentValue;
    }  
  }
    
  switch (entity)
  {
    case VERTEX:
      if (_verticesView != 0)
      {
        _meshList.push_back(_verticesView->getSelectedMesh());
        _transform = _verticesView->getSelectedTransform();
      
        _face = 0;
        _edge = 0;
        _vertex = _verticesView->getSelectedVertex();
      }  
      break;
     
    case EDGE:
      if (_edgesView != 0)
      {
        MFNode parentList = _edgesView->getSelectedVertex()->getParentList();
        _meshList.clear();
        for (MFNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
        {
          if (dynamic_cast<TemplateMesh<MData, VData, EData, FData, RW> *>(*it))
            _meshList.push_back(static_cast<TemplateMesh<MData, VData, EData, FData, RW> *>(*it));
        }
        
        _transform = _edgesView->getSelectedTransform();
      
        _face = 0;
        _edge = _edgesView->getSelectedEdge();
        _vertex = 0;
      }
      break;
    
    case FACE:    
      if (_facesView != 0)
      {
        _meshList.push_back(_facesView->getSelectedMesh());
        _transform = _facesView->getSelectedTransform();
      
        _face = _facesView->getSelectedFace();
        _edge = 0;
        _vertex = 0;
      }  
      break;
    
    case NIL:
      _meshList.clear();
      _face = 0;
      _edge = 0;
      _vertex = 0;
      _transform = SFMatrix34f::identity;
      break;
  }
  
  if (_verticesView != 0)
    _verticesView->setSelectedNodes(_meshList);
  if (_edgesView != 0)  
    _edgesView->setSelectedNodes(_meshList);
  if (_facesView != 0)  
    _facesView->setSelectedNodes(_meshList);
  
  for (std::list<View *>::iterator it = _viewList.begin(); it != _viewList.end(); ++it)
    (*it)->setSelectedNodes(_meshList);
}

template<class MData, class VData, class EData, class FData, bool RW>
MFNode MeshSelectionView<MData, VData, EData, FData, RW>::getSelectedNodes() const
{
  return _meshList;
}

template<class MData, class VData, class EData, class FData, bool RW>
void MeshSelectionView<MData, VData, EData, FData, RW>::addView(View *view)
{
  if (dynamic_cast<VerticesView<MData, VData, EData, FData, RW> *>(view) != 0)
  {
    _verticesView = static_cast<VerticesView<MData, VData, EData, FData, RW> *>(view);
    return;
  }
  
  if (dynamic_cast<EdgesView<MData, VData, EData, FData, RW> *>(view) != 0)
  {
    _edgesView = static_cast<EdgesView<MData, VData, EData, FData, RW> *>(view);
    return;
  }
  
  if (dynamic_cast<FacesView<MData, VData, EData, FData, RW> *>(view) != 0)
  {
    _facesView = static_cast<FacesView<MData, VData, EData, FData, RW> *>(view);
    return;
  }
  
  _viewList.push_back(view);
}

template<class MData, class VData, class EData, class FData, bool RW>
void MeshSelectionView<MData, VData, EData, FData, RW>::removeView(View *view)
{
  if (view == _verticesView)
  {
    _verticesView = 0;
    return; 
  }  
  
  if (view == _edgesView)
  {
    _edgesView = 0;
    return; 
  }  
  
  if (view == _facesView)
  {
    _facesView = 0;
    return; 
  }  
  
  MFNode::iterator it = std::find(_viewList.begin(), _viewList.end(), view);
  if (it != _viewList.end())
    _viewList.erase(it);
}
  
}
}
