#ifndef MESHNORMALSINVERSIONVIEW_H
#define MESHNORMALSINVERSIONVIEW_H

#include <MeshNormalsInversionDock.h>
#include <MESH_View.h>
#include <X3DTK/MESH/scenegraph.h>
#include <X3DTK/MESH/normalsinverter.h>
#include <X3DTK/MESH/normalsflipper.h>
#include <X3DTK/MESH/normalsupdater.h>
#include <GL/gl.h>
#include <X3DTK/kernel.h>
#include <ProgressDock.h>

namespace X3DTK {
namespace MESH {

/// View for the normals inversion, including a window to set parameters.

template<class MData, class VData, class EData, class FData, bool RW>
class NormalsInversionView : public View
{
public:
  /// Dock window for the graphical interface.
  MeshNormalsInversionDock *dock;

  /// Constructor.
  NormalsInversionView(QMainWindow *parent);
  /// Destructor.
  virtual ~NormalsInversionView();
  
  /// Hides the view.
  virtual void hide();
  /// Shows the view.
  virtual void show();
  
  /// Inverts the selection.
  void invert();
  /// Inverts the selection.
  void makeCoherent();
  /// Inverts the selection.
  void computeNormals();

  /// Sets the selected nodes to draw. If empty, then all the graph is drawn.
  void setSelectedNodes(const MFNode &selectedNodes);

private:
  TemplateNormalsInverter<MData, VData, EData, FData, RW> *_normalsInverter;
  TemplateNormalsFlipper<MData, VData, EData, FData, RW> *_normalsFlipper;  
  TemplateNormalsUpdater<MData, VData, EData, FData, RW> *_normalsUpdater;  
  ProgressDock *_progressDock;  
};

}
}

#include "MESH_NormalsInversionView.inl"

#endif
