namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
NormalsInversionView<MData, VData, EData, FData, RW>::NormalsInversionView(QMainWindow *parent) 
: View("normals inversion"), dock(new MeshNormalsInversionDock(parent))
{
  _normalsInverter = Singleton<TemplateNormalsInverter<MData, VData, EData, FData, RW> >::getInstance();
  _normalsFlipper = Singleton<TemplateNormalsFlipper<MData, VData, EData, FData, RW> >::getInstance();
  _normalsUpdater = Singleton<TemplateNormalsUpdater<MData, VData, EData, FData, RW> >::getInstance();
  _progressDock = new ProgressDock(parent, "work on progress...");
  
  RC::Signal::connect(dock, &MeshNormalsInversionDock::invert, this, &NormalsInversionView<MData, VData, EData, FData, RW>::invert);
  RC::Signal::connect(dock, &MeshNormalsInversionDock::makeCoherent, this, &NormalsInversionView<MData, VData, EData, FData, RW>::makeCoherent);
  RC::Signal::connect(dock, &MeshNormalsInversionDock::computeNormals, this, &NormalsInversionView<MData, VData, EData, FData, RW>::computeNormals);
}

template<class MData, class VData, class EData, class FData, bool RW>
NormalsInversionView<MData, VData, EData, FData, RW>::~NormalsInversionView()
{
  RC::Signal::disconnect(dock, &MeshNormalsInversionDock::invert, this, &NormalsInversionView<MData, VData, EData, FData, RW>::invert);
  RC::Signal::disconnect(dock, &MeshNormalsInversionDock::makeCoherent, this, &NormalsInversionView<MData, VData, EData, FData, RW>::makeCoherent);
  RC::Signal::disconnect(dock, &MeshNormalsInversionDock::computeNormals, this, &NormalsInversionView<MData, VData, EData, FData, RW>::computeNormals);
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsInversionView<MData, VData, EData, FData, RW>::hide()
{
  View::hide();
  dock->hide();
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsInversionView<MData, VData, EData, FData, RW>::show()
{
  View::show();
  dock->show();
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsInversionView<MData, VData, EData, FData, RW>::invert()
{
  _normalsInverter->invert(getScene());
  notify();
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsInversionView<MData, VData, EData, FData, RW>::makeCoherent()
{
  _normalsFlipper->flip(getScene());
  notify();
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsInversionView<MData, VData, EData, FData, RW>::computeNormals()
{
  _normalsUpdater->updateNormals(getScene());
  notify();
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsInversionView<MData, VData, EData, FData, RW>::setSelectedNodes(const MFNode &selectedNodes)
{
  _normalsFlipper->setSelectedNodes(selectedNodes);
  _normalsInverter->setSelectedNodes(selectedNodes);
  _normalsUpdater->setSelectedNodes(selectedNodes);
}

}
}
