namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
NormalsView<MData, VData, EData, FData, RW>::NormalsView(QMainWindow *parent) 
: View("normals"), dock(new MeshNormalsDock(parent)), _modified(true), _displayList(0), _amplitude(1.0f)
{
  _normalsDrawer = Singleton<TemplateNormalsDrawer<MData, VData, EData, FData, RW> >::getInstance();
  RC::Signal::connect(dock, &MeshNormalsDock::getValue, this, &NormalsView::setAmplitude);
}

template<class MData, class VData, class EData, class FData, bool RW>
NormalsView<MData, VData, EData, FData, RW>::~NormalsView()
{
  glDeleteLists(_displayList, 1);
  RC::Signal::disconnect(dock, &MeshNormalsDock::getValue, this, &NormalsView::setAmplitude);
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsView<MData, VData, EData, FData, RW>::hide()
{
  View::hide();
  dock->hide();
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsView<MData, VData, EData, FData, RW>::show()
{
  View::show();
  dock->show();
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsView<MData, VData, EData, FData, RW>::receiveNotification()
{
  _modified = true;
  glDeleteLists(_displayList, 1);
  _displayList = glGenLists(1);
  dock->init();
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsView<MData, VData, EData, FData, RW>::display()
{
  if (_modified)
  {
    glNewList(_displayList, GL_COMPILE_AND_EXECUTE);
    _normalsDrawer->draw(getScene(), _amplitude);
    glEndList();
    _modified = false;
  }  
  else
    glCallList(_displayList);
}

template<class MData, class VData, class EData, class FData, bool RW>
void NormalsView<MData, VData, EData, FData, RW>::setAmplitude(float amplitude)
{
  _amplitude = amplitude;
  _modified = true;
  RC::Signal::send(this, &View::updated);
}

}
}
