namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
StatisticsView<MData, VData, EData, FData, RW>::StatisticsView(QMainWindow *parent)
: View("statistics"), dock(new MeshStatisticsDock(parent))
{
  _statsCalculator = Singleton<TemplateStatsCalculator<MData, VData, EData, FData, RW> >::getInstance();
}

template<class MData, class VData, class EData, class FData, bool RW>
StatisticsView<MData, VData, EData, FData, RW>::~StatisticsView()
{
}

template<class MData, class VData, class EData, class FData, bool RW>  
void StatisticsView<MData, VData, EData, FData, RW>::hide()
{
  View::hide();
  dock->hide();
}

template<class MData, class VData, class EData, class FData, bool RW>
void StatisticsView<MData, VData, EData, FData, RW>::show()
{
  View::show();
  dock->show();
}

template<class MData, class VData, class EData, class FData, bool RW> 
void StatisticsView<MData, VData, EData, FData, RW>::receiveNotification()
{
  _statsCalculator->calculate(getScene());
  MFNode selection = _statsCalculator->getSelectedNodes();
  
  if (selection.empty())
  {
    dock->setFaces(_statsCalculator->getModelFaces());
    dock->setVertices(_statsCalculator->getModelVertices());
    dock->setEdges(_statsCalculator->getModelEdges()); 
  }
  else
  {
    unsigned int faces = 0;
    unsigned int vertices = 0;
    unsigned int edges = 0;
    
    for (MFNode::const_iterator it = selection.begin(); it != selection.end(); ++it)
    {
      faces += _statsCalculator->getFacesOf(*it);
      vertices += _statsCalculator->getVerticesOf(*it);
      edges += _statsCalculator->getEdgesOf(*it);
    }
    
    dock->setFaces(faces);
    dock->setVertices(vertices);
    dock->setEdges(edges);
  }  
}

template<class MData, class VData, class EData, class FData, bool RW>
void StatisticsView<MData, VData, EData, FData, RW>::display()
{
}

template<class MData, class VData, class EData, class FData, bool RW>
void StatisticsView<MData, VData, EData, FData, RW>::setSelectedNodes(const MFNode &selectedNodes)
{
  _statsCalculator->setSelectedNodes(selectedNodes);
}

}
}
