namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
VerticesView<MData, VData, EData, FData, RW>::VerticesView() 
: View("vertices"), _modified(true), _modifiedSelection(true), _displayList(0), _selectionDisplayList(0), _selectedMesh(0), _selectedVertex(0), _selectedTransform(SFMatrix34f::identity)
{
  _verticesDrawer = Singleton<TemplateVerticesDrawer<MData, VData, EData, FData, RW> >::getInstance();
}

template<class MData, class VData, class EData, class FData, bool RW>
VerticesView<MData, VData, EData, FData, RW>::~VerticesView()
{
  glDeleteLists(_displayList, 1);
  glDeleteLists(_selectionDisplayList, 1);
}

template<class MData, class VData, class EData, class FData, bool RW>
void VerticesView<MData, VData, EData, FData, RW>::receiveNotification()
{
  _modified = true;
  _modifiedSelection = true;
  glDeleteLists(_displayList, 1);
  glDeleteLists(_selectionDisplayList, 1);
  _displayList = glGenLists(1);
  _selectionDisplayList = glGenLists(1);
  _selectedMesh = 0;
  _selectedVertex = 0;
  _selectedTransform = SFMatrix34f::identity;
}

template<class MData, class VData, class EData, class FData, bool RW>
void VerticesView<MData, VData, EData, FData, RW>::display()
{
  if (_modified)
  {
    glNewList(_displayList, GL_COMPILE_AND_EXECUTE);
    _verticesDrawer->draw(getScene());
    glEndList();
    _modified = false;
  }  
  else
    glCallList(_displayList);
}
template<class MData, class VData, class EData, class FData, bool RW>
void VerticesView<MData, VData, EData, FData, RW>::initSelect()
{
  _selectedMesh = 0;
  _selectedVertex = 0;
  _selectedTransform = SFMatrix34f::identity;
}

template<class MData, class VData, class EData, class FData, bool RW>
void VerticesView<MData, VData, EData, FData, RW>::select()
{
  const int NB_HITS_MAX = 32768;
  // Prepare the selection mode
  static GLuint hits[NB_HITS_MAX];

  glSelectBuffer(NB_HITS_MAX, hits);
  glRenderMode(GL_SELECT);
  glInitNames();

  if (_modifiedSelection)
  {
    glNewList(_selectionDisplayList, GL_COMPILE_AND_EXECUTE);
    _verticesDrawer->draw(getScene(), true);
    glEndList();
    _modifiedSelection = false;
  }  
  else
    glCallList(_selectionDisplayList);

  glFlush();

  // Get the results 
  GLint nb_hits = glRenderMode(GL_RENDER);

  // Too many elements in selection buffer
 if (nb_hits < 0)  
    return;

 // No selection
  if (nb_hits == 0)
  {
    _selectedMesh = 0;
    _selectedVertex = 0;
    _selectedTransform = SFMatrix34f::identity;
    return;
  }
 
  //interpreting the results.
  _zMin = hits[1];
  int selectedMesh(hits[3]);  
  int selectedId(hits[4]);

  for (int i = 1; i < nb_hits; ++i)
  {
    // Prefer vertices over edges and edges over faces
    if (hits[i*5 + 1] < _zMin)
    {
      _zMin = hits[i*5 + 1];
      selectedMesh = hits[i*5 + 3];
      selectedId = hits[i*5 + 4];
    }
  }  

  _selectedMesh = _verticesDrawer->getMesh(selectedMesh);
  _selectedVertex = _selectedMesh->getVertices()[selectedId];
  _selectedTransform = _verticesDrawer->getMatrix(selectedMesh);
}

template<class MData, class VData, class EData, class FData, bool RW>
unsigned int VerticesView<MData, VData, EData, FData, RW>::getValue() const
{
  if (_selectedVertex != 0)
    return _zMin;
  return std::numeric_limits<unsigned int>::max();  
}

template<class MData, class VData, class EData, class FData, bool RW>
SFTemplateVertex<MData, VData, EData, FData, RW> *VerticesView<MData, VData, EData, FData, RW>::getSelectedVertex() const
{
  return _selectedVertex;
}

template<class MData, class VData, class EData, class FData, bool RW>
TemplateMesh<MData, VData, EData, FData, RW> *VerticesView<MData, VData, EData, FData, RW>::getSelectedMesh() const 
{
  return _selectedMesh;
}

template<class MData, class VData, class EData, class FData, bool RW>
const SFMatrix34f &VerticesView<MData, VData, EData, FData, RW>::getSelectedTransform() const 
{
  return _selectedTransform;
}

}
}
