#include "Viewer.h"
#include <qfiledialog.h>
#include <qmessagebox.h> 
#include <X3DTK/kernel.h>

using namespace X3DTK;
using namespace std;

Viewer::Viewer(const char *fileName)
: _fileName(fileName)
{
}

Viewer::~Viewer()
{
  scene.release();
}

void Viewer::init()
{
#ifdef GL_RESCALE_NORMAL
  glEnable(GL_RESCALE_NORMAL);
#endif  
  if (_fileName != 0)
    load(_fileName);
  else
    loadFromDialog();  
}

void Viewer::keyPressEvent(QKeyEvent *e)
{
  switch (e->key())
  {
    case Qt::Key_L : loadFromDialog(); break;
    default:         QGLViewer::keyPressEvent(e);
  }
}

void Viewer::load(const char *fileName)
{
  // Loads the file name and disabling the validation of the file by setting the 
  // 2nd parameter to false.
  scene.load(fileName, false);
  
  // QGLViewer settings.
  setSceneBoundingBox(scene.getBBoxMin().f_data(), scene.getBBoxMax().f_data());
  showEntireScene();
}

void Viewer::loadFromDialog()
{
  QString fileName = QFileDialog::getOpenFileName("", "X3D files (*.x3d *.X3D);;All files (*)", this);
  
  // In case of Cancel
  if (fileName.isEmpty())
    return;
    
  load(fileName);
}

void Viewer::draw()
{
  // Draws the scene.
  scene.draw();
}

QString Viewer::helpString() const
{
  QString message("");
  
  message += "<b>L</b>" + QString(" loads a new file<br>");
  
  message += QGLViewer::helpString();
  
  return message;
}

void Viewer::help() const
{
  QMessageBox *mb = new QMessageBox("help", helpString(), QMessageBox::NoIcon,QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton,QMessageBox::NoButton, NULL, "Help", false,Qt::WStyle_DialogBorder | Qt::WType_Dialog | Qt::WDestructiveClose);
  mb->show();
}
